require "card_game/player"
require "card_game/card"

module CardGame
    describe Player do
        
        before do
            @initial_rank = 1
            @initial_gold = 200
            @initial_score = 100
            @player = Player.new("moe",@initial_rank, @initial_gold,@initial_score)
            @player2 = Player.new("jim",@initial_rank, @initial_gold,@initial_score)
            
            
        end
        
        it "should have a name attribute" do
            expect(@player.name).to eq("Moe")
        end
        
        it "should have a rank attribute" do
            expect(@player.rank).to eq(@initial_rank)
        end
        
        it "should have a gold attribute" do
            expect(@player.gold).to eq(@initial_gold)
        end
        
        it "should have a score attribute" do
            expect(@player.score).to eq(@initial_score)
        end
        
        it "should increase rank by 1 by setting it to 1" do
            @player.set_rank(1)
            expect(@player.rank).to eq(1)
        end
        
        it "should add gold by 100" do
            #@player.add_gold
            #expect(@player.gold).to eq(@initial_gold - 8)
        end
        
        it "should deduct gold by 100" do
            expect(@player.gold).to eq(200)
            @player.deduct_gold
            expect(@player.gold).to eq(192)
        end
        
        it "should add score by 100" do
            @player.add_score(1)
            expect(@player.score).to eq(@initial_score + 100)
        end
        
        it "should deduct score by 50" do
            @player.deduct_score
            expect(@player.score).to eq(@initial_score - 50)
        end
        
        it "should add cards in player's deck" do
            cards = Cards::CARDS
            cards.each do |card|
                @player.add_card(card)
            end
            
            expect(@player.deck.empty?).to eq(false)
            expect(@player.deck.length).to eq(3)
            
        end
        
        context "Player 1 and player 2 has 2 random cards" do
            before do
                #@player.add_card(Card.new(:giant,3000,100))
                #@player.add_card(Card.new(:goblin,1000,150))
                #@player2.add_card(Card.new(:goblin,1000,150))
                #@player2.add_card(Card.new(:sniffers, 2000, 80))
                #@player2.add_card(Card.new(:honkerdown,4000,80))
                
                #@player.current_card_to_play
                #@player2.current_card_to_play
                
            end
            
            it "current card should take a card from the deck" do
                @player.add_card(Card.new(:giant,3000,100))
                @player.current_card_to_play
                expect(@player.current_card.name.to_s).to eq("giant")
            end
            
            it "should not pick out card where health is zero" do
                @player.add_card(Card.new(:giant,3000,100))
                @player.current_card_to_play
                
                @player.add_card(Card.new(:bowman,1000, 80))
                @player.current_card.health = 0
                @player.current_card_to_play
                
                
                expect(@player.current_card.name.to_s).to eq("bowman")
                
                #@player.current_card_to_play
                
            end
            
        end
        
        context "Player1 has a giant card and player 2 has a goblin card" do
            
            before do
                @player.add_card(Card.new(:giant,3000,100))
                @player2.add_card(Card.new(:goblin,1000,150))
                
                @player.current_card_to_play
                @player2.current_card_to_play
                
            end
            
            it "current card should not be nil" do
                expect(@player.current_card).not_to be(false)
            end
            
            it "should output card correctly" do
                expect(@player.current_card.name.to_s).to eq("giant")
                expect(@player2.current_card.name.to_s).to eq("goblin")
            end
            
            it "should have been attacked by another player" do
                allow_any_instance_of(Die).to receive(:roll).and_return(0)
                @player.got_attack?(@player2)
                expect(@player.current_card.health).to eq(3000 - 150)
            end
            
            it "should have a 25% of blocking the attack" do
                allow_any_instance_of(Die).to receive(:roll).and_return(1)
                @player.got_attack?(@player2)
                expect(@player.current_card.health).to eq(3000 - (150/4))
            end
            
            it "should have a 50% of blocking the attack" do
                allow_any_instance_of(Die).to receive(:roll).and_return(2)
                @player.got_attack?(@player2)
                expect(@player.current_card.health).to eq(3000 - (150/2))
            end
            
            it "should have a 75% of blocking the attack" do
                allow_any_instance_of(Die).to receive(:roll).and_return(3)
                @player.got_attack?(@player2)
                expect(@player.current_card.health).to eq(3000 - (3*(150/4)))
            end
            
            it "should have successfully blocked the attack" do
                allow_any_instance_of(Die).to receive(:roll).and_return(4)
                @player.got_attack?(@player2)
                expect(@player.current_card.health).to eq(3000)            
            end
            
        end
        
        
        
    end
end